#include <pjsr/ColorSpace.jsh>
#include <pjsr/UndoFlag.jsh>
#include <pjsr/StdCursor.jsh>
#include <pjsr/Sizer.jsh>
#include <pjsr/FrameStyle.jsh>
#include <pjsr/TextAlign.jsh>
#include <pjsr/NumericControl.jsh>
#include <pjsr/FontFamily.jsh>
#include <pjsr/SampleType.jsh>

// Shadows clipping point measured in sigma units from the main histogram peak.
#define SHADOWS_CLIP -1.25

// Target background in the [0,1] range.
#define TARGET_BKG    0.25

// Whether to apply the same STF to all channels, or treat each channel separately.
#define RGB_LINK      true



/*
 * Find a midtones balance value that transforms v1 into v0 through a midtones
 * transfer function (MTF), within the specified tolerance eps.
 */
function findMidtonesBalance( v0, v1, eps )
{
   if ( v1 <= 0 )
      return 0;

   if ( v1 >= 1 )
      return 1;

   v0 = Math.range( v0, 0.0, 1.0 );

   if ( eps )
      eps = Math.max( 1.0e-15, eps );
   else
      eps = 5.0e-05;

   var m0, m1;
   if ( v1 < v0 )
   {
      m0 = 0;
      m1 = 0.5;
   }
   else
   {
      m0 = 0.5;
      m1 = 1;
   }

   for ( ;; )
   {
      var m = (m0 + m1)/2;
      var v = Math.mtf( m, v1 );

      if ( Math.abs( v - v0 ) < eps )
         return m;

      if ( v < v0 )
         m1 = m;
      else
         m0 = m;
   }
}




function HaRGBCombinationData()
{
this.originalView= ImageWindow.activeWindow.mainView;



   this.Preview= new ImageWindow(this.originalView.image.width,
                                 this.originalView.image.height,
                                 3,
                                 32,
                                 true,
                                 true,
                                 "Preview");

   this.Preview.mainView.beginProcess(UndoFlag_NoSwapFile);
   this.Preview.mainView.image.fill( 0 );
   this.Preview.mainView.endProcess();
   this.Preview.zoomFactor=12;
   this.Preview.fitWindow();
   this.Preview.zoomToOptimalFit();
   this.Preview.show();

   this.HaRGBCombinationView= new ImageWindow(this.originalView.image.width,
                                 this.originalView.image.height,
                                 3,
                                 32,
                                 true,
                                 true,
                                 "HaRGBCombination");

   this.HaRGBCombinationView.mainView.beginProcess(UndoFlag_NoSwapFile);
   this.HaRGBCombinationView.mainView.image.fill( 0 );
   this.HaRGBCombinationView.mainView.endProcess();
   this.HaRGBCombinationView.zoomFactor=12;
   this.HaRGBCombinationView.fitWindow();
   this.HaRGBCombinationView.zoomToOptimalFit();




   this.HaEmissionView= new ImageWindow(this.originalView.image.width,
                                 this.originalView.image.height,
                                 1,
                                 32,
                                 true,
                                 false,
                                 "HaEmission");

   this.HaEmissionView.mainView.beginProcess(UndoFlag_NoSwapFile);
   this.HaEmissionView.mainView.image.fill( 0 );
   this.HaEmissionView.mainView.endProcess();


   this.RView= new ImageWindow(this.originalView.image.width,
                                 this.originalView.image.height,
                                 1,
                                 32,
                                 true,
                                 false,
                                 "R");

   this.RView.mainView.beginProcess(UndoFlag_NoSwapFile);
   this.RView.mainView.image.fill( 0 );
   this.RView.mainView.endProcess();

   this.GView= new ImageWindow(this.originalView.image.width,
                                 this.originalView.image.height,
                                 1,
                                 32,
                                 true,
                                 false,
                                 "G");

   this.GView.mainView.beginProcess(UndoFlag_NoSwapFile);
   this.GView.mainView.image.fill( 0 );
   this.GView.mainView.endProcess();

   this.BView= new ImageWindow(this.originalView.image.width,
                                 this.originalView.image.height,
                                 1,
                                 32,
                                 true,
                                 false,
                                 "B");

   this.BView.mainView.beginProcess(UndoFlag_NoSwapFile);
   this.BView.mainView.image.fill( 0 );
   this.BView.mainView.endProcess();



this.sourceHaView=null;
this.sourceHaBandwidth=7;
this.sourceHaMultiplication=1.20;

this.sourceRGBView=null;
this.sourceRGBBandwidth=75;

this.RGBImageSTF=null;
this.HaRGBImageSTF=null;

this.CCBgView=null;
this.CCWhiteView=null;
this.StructureDetection=true;
this.numberOfLayers=3;
this.TSCNR=true;
this.CC=false;

this.CSTF=true;
this.Changed=true;

this.CRescale=false;

   this.abort = false;
   this.busy = false;
   this.terminate = false;

}





function MyDialog()
{
   this.__base__ = Dialog;
   this.__base__();



this.ApplyAutoSTF = function( view, shadowsClipping, targetBackground, rgbLinked )
{
   var stf = new ScreenTransferFunction;

   var n = view.image.isColor ? 3 : 1;

   if ( rgbLinked )
   {
      /*
       * Try to find how many channels look as channels of an inverted image.
       * We know a channel has been inverted because the main histogram peak is
       * located over the right-hand half of the histogram. Seems simplistic
       * but this is consistent with astronomical images.
       */
      var invertedChannels = 0;
      for ( var c = 0; c < n; ++c )
      {
         view.image.selectedChannel = c;
         if ( view.image.median() > 0.5 )
            ++invertedChannels;
      }
      view.image.resetSelections();

      if ( invertedChannels < n )
      {
         // Noninverted image

         var c0 = 0;
         var m = 0;
         for ( var c = 0; c < n; ++c )
         {
            view.image.selectedChannel = c;
            var median = view.image.median();
            var avgDev = view.image.avgDev();
            c0 += median + shadowsClipping*avgDev;
            m  += median;
         }
         view.image.resetSelections();
         c0 = Math.range( c0/n, 0.0, 1.0 );
         m = findMidtonesBalance( targetBackground, m/n - c0 );

         stf.STF = [ // c0, c1, m, r0, r1
                     [c0, 1, m, 0, 1],
                     [c0, 1, m, 0, 1],
                     [c0, 1, m, 0, 1],
                     [0, 1, 0.5, 0, 1] ];
      }
      else
      {
         // Inverted image

         var c1 = 0;
         var m = 0;
         for ( var c = 0; c < n; ++c )
         {
            view.image.selectedChannel = c;
            var median = view.image.median();
            var avgDev = view.image.avgDev();
            m  += median;
            c1 += median - shadowsClipping*avgDev;
         }
         view.image.resetSelections();
         c1 = Math.range( c1/n, 0.0, 1.0 );
         m = 1 - findMidtonesBalance( targetBackground, c1 - m/n );

         stf.STF = [ // c0, c1, m, r0, r1
                     [0, c1, m, 0, 1],
                     [0, c1, m, 0, 1],
                     [0, c1, m, 0, 1],
                     [0, 1, 0.5, 0, 1] ];
      }
   }
   else
   {
      var A = [ // c0, c1, m, r0, r1
               [0, 1, 0.5, 0, 1],
               [0, 1, 0.5, 0, 1],
               [0, 1, 0.5, 0, 1],
               [0, 1, 0.5, 0, 1] ];

      for ( var c = 0; c < n; ++c )
      {
         view.image.selectedChannel = c;
         var median = view.image.median();
         var avgDev = view.image.avgDev();

         if ( median < 0.5 )
         {
            // Noninverted channel
            var c0 = Math.range( median + shadowsClipping*avgDev, 0.0, 1.0 );
            var m  = findMidtonesBalance( targetBackground, median - c0 );
            A[c] = [c0, 1, m, 0, 1];
         }
         else
         {
            // Inverted channel
            var c1 = Math.range( median - shadowsClipping*avgDev, 0.0, 1.0 );
            var m  = 1 - findMidtonesBalance( targetBackground, c1 - median() );
            A[c] = [0, c1, m, 0, 1];
         }
      }

      stf.STF = A;

      view.image.resetSelections();
   }

   console.writeln( "<end><cbr/><br/><b>", view.fullId, "</b>:" );
   for ( var c = 0; c < n; ++c )
   {
      console.writeln( "channel #", c );
      console.writeln( format( "c0 = %.6f", stf.STF[c][0] ) );
      console.writeln( format( "m  = %.6f", stf.STF[c][2] ) );
      console.writeln( format( "c1 = %.6f", stf.STF[c][1] ) );
   }

   stf.executeOn( view );

   console.writeln( "<end><cbr/><br/>" );
   return stf
}










   this.applySTF=function( img, stf )
   {
      var HT = new HistogramTransformation;
      HT.H = [[stf[0][1], stf[0][0], stf[0][2], stf[0][3], stf[0][4]],
              [stf[1][1], stf[1][0], stf[1][2], stf[1][3], stf[1][4]],
              [stf[2][1], stf[2][0], stf[2][2], stf[2][3], stf[2][4]],
              [ 0, 0.5, 1, 0, 1]];

      var wtmp = new ImageWindow( 1, 1, 1,
         img.bitsPerSample, img.sampleType == SampleType_Real, img.isColor, "tmpSTFWindow" );
      var v = wtmp.mainView;

      v.beginProcess( UndoFlag_NoSwapFile );
      v.image.assign( img );
      v.endProcess();

      HT.executeOn( v, false ); // no swap file
      var image=v.image;
      var result=new Image( image.width, image.height,
                            image.numberOfChannels, image.colorSpace,
                            image.bitsPerSample, image.sampleType);
      result.assign(v.image);
      wtmp.forceClose();
      return result;
   };

   this.showOriginal = function()
   {

if (data.sourceRGBView==null){
var msg = new MessageBox( "Please select a source RGB image.",
                                "HaRGB Combination Script", StdIcon_Error, StdButton_Ok );
      msg.execute();
return;
}



   if (data.CSTF){
   if (data.RGBImageSTF==null){
   console.show();
   this.ApplyAutoSTF( data.sourceRGBView, SHADOWS_CLIP, TARGET_BKG, RGB_LINK );
   data.RGBImageSTF = this.applySTF(data.sourceRGBView.image,data.sourceRGBView.stf);

   }
   data.Preview.mainView.beginProcess(UndoFlag_NoSwapFile);
   data.Preview.mainView.image.assign(data.RGBImageSTF);
   data.Preview.mainView.endProcess();
   this.ScrollControl.repaint();
   console.hide();
   }else{
   data.Preview.mainView.beginProcess(UndoFlag_NoSwapFile);
   data.Preview.mainView.image.assign(data.sourceRGBView.image);
   data.Preview.mainView.endProcess();
   this.ScrollControl.repaint();
   console.hide();

   }

}

   this.showHaRGB = function()
   {

if (data.sourceRGBView==null || data.sourceHaView==null ){
var msg = new MessageBox( "Please select a source Ha and RGB images.",
                                "HaRGB Combination Script", StdIcon_Error, StdButton_Ok );
      msg.execute();
return;
}

if (data.CC){
if (data.CCBgView==null || data.CCWhiteView==null ){
var msg = new MessageBox( "Please select a background and white balance previews.",
                                "HaRGB Combination Script", StdIcon_Error, StdButton_Ok );
      msg.execute();
return;
}
}

data.HaRGBCombinationView.mainView.stf=[[0.5,1,0,0,1],[0.5,1,0,0,1],[0.5,1,0,0,1],[0.5,1,0,0,1]];

if (data.Changed){
console.show();
var P = new PixelMath;
P.expression = "(("+data.sourceHaView.id+"*"+data.sourceRGBBandwidth+")-("+data.sourceRGBView.id+"*"+data.sourceHaBandwidth+"))/("+data.sourceRGBBandwidth+"-"+data.sourceHaBandwidth+")";
P.expression1 = "";
P.expression2 = "";
P.expression3 = "";
P.useSingleExpression = true;
P.symbols = "";
P.use64BitWorkingImage = false;
P.rescale = data.CRescale;
P.rescaleLower = 0.0000000000;
P.rescaleUpper = 1.0000000000;
P.truncate = true;
P.truncateLower = 0.0000000000;
P.truncateUpper = 1.0000000000;
P.createNewImage = false;
P.newImageId = "";
P.newImageWidth = 0;
P.newImageHeight = 0;
P.newImageAlpha = false;
P.newImageColorSpace = PixelMath.prototype.SameAsTarget;
P.newImageSampleFormat = PixelMath.prototype.SameAsTarget;


   data.HaEmissionView.mainView.beginProcess(UndoFlag_NoSwapFile);
   P.executeOn(data.HaEmissionView.mainView);
   data.HaEmissionView.mainView.endProcess();


   data.HaRGBCombinationView.mainView.beginProcess(UndoFlag_NoSwapFile);
   data.HaRGBCombinationView.mainView.image.assign(data.sourceRGBView.image);
   data.HaRGBCombinationView.mainView.endProcess();

var P = new PixelMath;
P.expression = data.sourceRGBView.id+"[0]+(HaEmission-med(HaEmission))*"+data.sourceHaMultiplication;
P.expression1 = "";
P.expression2 = "";
P.expression3 = "";
P.useSingleExpression = true;
P.symbols = "";
P.use64BitWorkingImage = false;
P.rescale = data.CRescale;
P.rescaleLower = 0.0000000000;
P.rescaleUpper = 1.0000000000;
P.truncate = true;
P.truncateLower = 0.0000000000;
P.truncateUpper = 1.0000000000;
P.createNewImage = false;
P.newImageId = "";
P.newImageWidth = 0;
P.newImageHeight = 0;
P.newImageAlpha = false;
P.newImageColorSpace = PixelMath.prototype.SameAsTarget;
P.newImageSampleFormat = PixelMath.prototype.SameAsTarget;



   data.RView.mainView.beginProcess(UndoFlag_NoSwapFile);
   P.executeOn(data.RView.mainView);
   data.RView.mainView.endProcess();


var P = new LinearFit;
P.referenceViewId = "R";
P.rejectLow = 0.000000;
P.rejectHigh = 0.920000;

   data.HaEmissionView.mainView.beginProcess(UndoFlag_NoSwapFile);
   P.executeOn(data.HaEmissionView.mainView);
   data.HaEmissionView.mainView.endProcess();




var P = new PixelMath;
P.expression = "max(HaEmission,R)";
P.expression1 = "";
P.expression2 = "";
P.expression3 = "";
P.useSingleExpression = true;
P.symbols = "";
P.use64BitWorkingImage = false;
P.rescale = data.CRescale;
P.rescaleLower = 0.0000000000;
P.rescaleUpper = 1.0000000000;
P.truncate = true;
P.truncateLower = 0.0000000000;
P.truncateUpper = 1.0000000000;
P.createNewImage = false;
P.newImageId = "";
P.newImageWidth = 0;
P.newImageHeight = 0;
P.newImageAlpha = false;
P.newImageColorSpace = PixelMath.prototype.SameAsTarget;
P.newImageSampleFormat = PixelMath.prototype.SameAsTarget;



   data.RView.mainView.beginProcess(UndoFlag_NoSwapFile);
   P.executeOn(data.RView.mainView);
   data.RView.mainView.endProcess();



var P = new PixelMath;
P.expression = data.sourceRGBView.id+"[1]";
P.expression1 = "";
P.expression2 = "";
P.expression3 = "";
P.useSingleExpression = true;
P.symbols = "";
P.use64BitWorkingImage = false;
P.rescale = false;
P.rescaleLower = 0.0000000000;
P.rescaleUpper = 1.0000000000;
P.truncate = true;
P.truncateLower = 0.0000000000;
P.truncateUpper = 1.0000000000;
P.createNewImage = false;
P.newImageId = "";
P.newImageWidth = 0;
P.newImageHeight = 0;
P.newImageAlpha = false;
P.newImageColorSpace = PixelMath.prototype.SameAsTarget;
P.newImageSampleFormat = PixelMath.prototype.SameAsTarget;

   data.GView.mainView.beginProcess(UndoFlag_NoSwapFile);
   P.executeOn(data.GView.mainView);
   data.GView.mainView.endProcess();
var P = new PixelMath;
P.expression = data.sourceRGBView.id+"[2]";
P.expression1 = "";
P.expression2 = "";
P.expression3 = "";
P.useSingleExpression = true;
P.symbols = "";
P.use64BitWorkingImage = false;
P.rescale = false;
P.rescaleLower = 0.0000000000;
P.rescaleUpper = 1.0000000000;
P.truncate = true;
P.truncateLower = 0.0000000000;
P.truncateUpper = 1.0000000000;
P.createNewImage = false;
P.newImageId = "";
P.newImageWidth = 0;
P.newImageHeight = 0;
P.newImageAlpha = false;
P.newImageColorSpace = PixelMath.prototype.SameAsTarget;
P.newImageSampleFormat = PixelMath.prototype.SameAsTarget;
   data.BView.mainView.beginProcess(UndoFlag_NoSwapFile);
   P.executeOn(data.BView.mainView);
   data.BView.mainView.endProcess();


   var P = new ChannelCombination;
   P.colorSpace = ChannelCombination.prototype.RGB;
   P.channels = [ // enabled, id
   [true, "R"],
   [true, "G"],
   [true, "B"]
   ];


   data.HaRGBCombinationView.mainView.beginProcess(UndoFlag_NoSwapFile);
   P.executeOn(data.HaRGBCombinationView.mainView);
   if (data.CC){
   if (data.CCBgView!=null){
   data.HaRGBCombinationView.createPreview (window.previewRect(data.CCBgView),data.CCBgView.id);
   }
   if (data.CCWhiteView!=null){
   if (data.CCWhiteView.isPreview){
    data.HaRGBCombinationView.createPreview (window.previewRect(data.CCWhiteView),data.CCWhiteView.id);
   }
   }
   }
   data.HaRGBCombinationView.mainView.endProcess();


if (data.CC){
   var P = new BackgroundNeutralization;
   P.backgroundReferenceViewId = "HaRGBCombination->"+data.CCBgView.id;
   P.backgroundLow = 0.000000;
   P.backgroundHigh = 0.100000;
   P.useROI = false;
   P.roiX0 = 0;
   P.roiY0 = 0;
   P.roiX1 = 0;
   P.roiY1 = 0;
   P.mode = BackgroundNeutralization.prototype.RescaleAsNeeded;
   P.targetBackground = 0.010000;

   data.HaRGBCombinationView.mainView.beginProcess(UndoFlag_NoSwapFile);
   P.executeOn(data.HaRGBCombinationView.mainView);
   data.HaRGBCombinationView.mainView.endProcess();

   var WhiteRef="HaRGBCombination";
   var BgRef="HaRGBCombination";

   if (data.CCWhiteView.isPreview){
   WhiteRef="HaRGBCombination->"+data.CCWhiteView.id;
   }

   if (data.CCBgView.isPreview){
   BgRef="HaRGBCombination->"+data.CCBgView.id;
   }

   var P = new ColorCalibration;
   P.whiteReferenceViewId = WhiteRef;
   P.whiteLow = 0.00000;
   P.whiteHigh = 0.90000;
   P.whiteUseROI = false;
   P.whiteROIX0 = 0;
   P.whiteROIY0 = 0;
   P.whiteROIX1 = 0;
   P.whiteROIY1 = 0;
   P.structureDetection = data.StructureDetection;
   P.structureLayers = data.numberOfLayers;
   P.noiseLayers = 1;
   P.manualWhiteBalance = false;
   P.manualRedFactor = 1.0000;
   P.manualGreenFactor = 1.0000;
   P.manualBlueFactor = 1.0000;
   P.backgroundReferenceViewId = BgRef;
   P.backgroundLow = 0.000000;
   P.backgroundHigh = 0.100000;
   P.backgroundUseROI = false;
   P.backgroundROIX0 = 0;
   P.backgroundROIY0 = 0;
   P.backgroundROIX1 = 0;
   P.backgroundROIY1 = 0;
   P.outputWhiteReferenceMask = false;
   P.outputBackgroundReferenceMask = false;

   data.HaRGBCombinationView.mainView.beginProcess(UndoFlag_NoSwapFile);
   P.executeOn(data.HaRGBCombinationView.mainView);
   data.HaRGBCombinationView.mainView.endProcess();

if (data.TSCNR){
   var P = new SCNR;
   P.amount = 1.00;
   P.protectionMethod = SCNR.prototype.AverageNeutral;
   P.colorToRemove = SCNR.prototype.Green;
   P.preserveLightness = true;
   data.HaRGBCombinationView.mainView.beginProcess(UndoFlag_NoSwapFile);
   P.executeOn(data.HaRGBCombinationView.mainView);
   data.HaRGBCombinationView.mainView.endProcess();
}
}

if (data.CSTF){
   this.ApplyAutoSTF( data.HaRGBCombinationView.mainView, SHADOWS_CLIP, TARGET_BKG, RGB_LINK );
   data.HaRGBImageSTF = this.applySTF(data.HaRGBCombinationView.mainView.image,data.HaRGBCombinationView.mainView.stf);
}
console.hide();
data.Changed=false;

}

if (data.CSTF){
   data.Preview.mainView.beginProcess(UndoFlag_NoSwapFile);
   data.Preview.mainView.image.assign(data.HaRGBImageSTF);
   data.Preview.mainView.endProcess();
   this.ScrollControl.repaint();
}else{
   data.Preview.mainView.beginProcess(UndoFlag_NoSwapFile);
   data.Preview.mainView.image.assign(data.HaRGBCombinationView.mainView.image);
   data.Preview.mainView.endProcess();
   this.ScrollControl.repaint();
}
}






   this.onMouseWheel = function(Dx,Dy, MouseWheel)
   {
      if (MouseWheel >0) data.Preview.zoomOut()
      else data.Preview.zoomIn()
      this.ScrollControl.repaint();
   }


   this.ScrollControl = new Control( this );
   with ( this.ScrollControl )
   {

      var w = data.Preview.mainView.image.width;
      var h = data.Preview.mainView.image.height;
      var Size=data.Preview.viewportWidth/1.5;
      setMinSize( Size, Size*h/w );
      var k=1/w*Size;
      cursor = new Cursor( StdCursor_CirclePlus );



      onPaint = function()
      {
         // extract viewing parts of image dest_(x0 x1 y0 y1)
         var Zoom=data.Preview.viewportWidth/w;
         var Dx0=data.Preview.viewportPosition.x/Zoom;
         var Dy0=data.Preview.viewportPosition.y/Zoom;
         var Dx1=data.Preview.visibleViewportRect.x1/Zoom;
         var Dy1=data.Preview.visibleViewportRect.y1/Zoom;
         if (Dx1 > w) Dx1=w;
         if (Dy1 > h) Dy1=h;

         var x0=Dx0*k;
         var y0=Dy0*k;
         var x1=x0+Dx1*k;
         var y1=y0+Dy1*k;



         var G = new Graphics( this );
         G.drawScaledBitmap( this.boundsRect, data.Preview.mainView.image.render() );
         G.pen = new Pen( 0xFF00FF00 ); //Green
         G.drawRect(x0,y0,x1,y1);
         G.end()
         gc();
      }

      onMousePress = function(x,y)
      {
         data.Preview.setViewport( x*w/Size, y*w/Size );
         repaint();
      }

      onMouseMove = function(x,y)
      {
         data.Preview.setViewport( x*w/Size, y*w/Size );
         repaint();
      }
   }







  //---------------------------------------------------------------------------------------
   // buttons


var labelWidth1 = this.font.width( "--------------------------------------------" + 'T' );



   this.ZoomToFit_Button = new PushButton( this );
   with ( this.ZoomToFit_Button )
   {
      text = "Zoom To Fit";
      onClick = function()
      {
         data.Preview.zoomToFit();
         parent.parent.ScrollControl.repaint();

      }
   }


   this.fitWindow_Button = new PushButton( this );
   with ( this.fitWindow_Button )
   {
      text = "Fit View";
      onClick = function()
      {
         data.Preview.fitWindow();
         parent.parent.ScrollControl.repaint();
      }
   }


   this.zoomToOptimalFit_Button = new PushButton( this );
   with ( this.zoomToOptimalFit_Button )
   {
      text = "ZoomToOptimalFit";
      onClick = function()
      {
         data.Preview.zoomToOptimalFit();
         parent.parent.ScrollControl.repaint();
      }
   }







   this.okButton = new PushButton( this );
   this.okButton.text = "OK";
   this.okButton.onClick = function()
   {
      data.HaRGBCombinationView.deletePreviews();
      data.HaRGBCombinationView.show();
      data.Preview.forceClose();
      data.RView.forceClose();
      data.GView.forceClose();
      data.BView.forceClose();
      data.HaEmissionView.forceClose();
      this.dialog.ok();
   }

   this.cancelButton = new PushButton( this );
   this.cancelButton.text = "Cancel";
   this.cancelButton.onClick = function()
   {
      data.HaRGBCombinationView.deletePreviews();
      data.HaRGBCombinationView.forceClose();
      data.Preview.forceClose();
      data.RView.forceClose();
      data.GView.forceClose();
      data.BView.forceClose();
      data.HaEmissionView.forceClose();
      this.dialog.cancel();
   }

   this.zoombuttonssizer = new HorizontalSizer;
   with ( this.zoombuttonssizer )
   {
      margin = 0;
      spacing = 6;
      add( this.ZoomToFit_Button );
      add( this.fitWindow_Button );
      add( this.zoomToOptimalFit_Button );
   }



this.PWGroupBox = new GroupBox( this );
   with ( this.PWGroupBox )
   {

      title = "Window Preview Control";
      sizer = new VerticalSizer;
      sizer.margin = 6;
      sizer.spacing = 4;
      sizer.add( this.ScrollControl );
      sizer.add( this.zoombuttonssizer );

  }

   this.sourceHaImage_Label = new Label( this );
   this.sourceHaImage_Label.minWidth = labelWidth1; // align with labels inside group boxes below
   this.sourceHaImage_Label.text = "Source Ha image:";
   this.sourceHaImage_Label.textAlignment = TextAlign_Right|TextAlign_VertCenter;

   this.sourceHaImage_ViewList = new ViewList( this );
   this.sourceHaImage_ViewList.minWidth = 200;
   this.sourceHaImage_ViewList.getAll(); // include main views as well as previews
   this.sourceHaImage_ViewList.toolTip = "Select the image with the Ha emmision.";
   this.sourceHaImage_ViewList.onViewSelected = function( view )
   {
      data.Changed=true;
      data.sourceHaView = view;
   };

   this.sourceHaImage_Sizer = new HorizontalSizer;
   this.sourceHaImage_Sizer.spacing = 4;
   this.sourceHaImage_Sizer.add( this.sourceHaImage_Label );
   this.sourceHaImage_Sizer.add( this.sourceHaImage_ViewList, 100 );

   this.sourceHaControl = new NumericControl( this );
   with ( this.sourceHaControl )
   {
      setRange( 0, 200 );
      label.text = "HAlpha Bandwidth:";
      label.minWidth = labelWidth1;
      slider.setRange( 0, 200 );
      slider.minWidth = 200;
       setPrecision (0);
      setValue( data.sourceHaBandwidth);
      toolTip = "<p>HAlpha Bandwith.</p>";

      onValueUpdated = function( value )
      {
         data.Changed=true;
         data.sourceHaBandwidth = value;
      };
   }

   this.sourceHaMultiplicationControl = new NumericControl( this );
   with ( this.sourceHaMultiplicationControl )
   {
      setRange( 0.0, 10.00 );
      label.text = "HAlpha multiplication factor:";
      label.minWidth = labelWidth1;
      slider.setRange( 0, 100 );
      slider.minWidth = 200;
      setValue( data.sourceHaMultiplication);
      setPrecision (2);
      toolTip = "<p>HAlpha multiplication factor.</p>";

      onValueUpdated = function( value )
      {
         data.Changed=true;
         data.sourceHaMultiplication = value;
      };
   }


this.sourceRGBImage_Label = new Label( this );
   this.sourceRGBImage_Label.minWidth = labelWidth1; // align with labels inside group boxes below
   this.sourceRGBImage_Label.text = "Source RGB image:";
   this.sourceRGBImage_Label.textAlignment = TextAlign_Right|TextAlign_VertCenter;

   this.sourceRGBImage_ViewList = new ViewList( this );
   this.sourceRGBImage_ViewList.minWidth = 200;
   this.sourceRGBImage_ViewList.getAll(); // include main views as well as previews
   this.sourceRGBImage_ViewList.toolTip = "Select the RGB image.";
   this.sourceRGBImage_ViewList.onViewSelected = function( view )
   {
      data.Changed=true;
      data.sourceRGBView = view;
   };

   this.sourceRGBImage_Sizer = new HorizontalSizer;
   this.sourceRGBImage_Sizer.spacing = 4;
   this.sourceRGBImage_Sizer.add( this.sourceRGBImage_Label );
   this.sourceRGBImage_Sizer.add( this.sourceRGBImage_ViewList, 100 );

   this.sourceRGBControl = new NumericControl( this );
   with ( this.sourceRGBControl )
   {
      setRange( 0, 200 );
      label.text = "RGB Bandwidth:";
      label.minWidth = labelWidth1;
      slider.setRange( 0, 900 );
      slider.minWidth = 200;
      setPrecision (0);
      setValue( data.sourceRGBBandwidth);
      toolTip = "<p>RGB Filters Bandwidth</p>";

      onValueUpdated = function( value )
      {
         data.Changed=true;
         data.sourceRGBBandwidth = value;
      };
   }

   this.Rescale_CheckBox = new CheckBox( this );
   this.Rescale_CheckBox.text = "Stretched Images";
   this.Rescale_CheckBox.checked = data.CRescale;
   this.Rescale_CheckBox.toolTip =
      "<p>If this option is selected, the script will rescale "
      + "pixelmath expressions for non linear images.</p>";
   this.Rescale_CheckBox.onCheck = function( checked )
   {
      data.Changed=true;
      data.CRescale = checked;
   };

this.ChannelsGroupBox = new GroupBox( this );
   with ( this.ChannelsGroupBox )
   {

      title = "Channels Control";
      sizer = new VerticalSizer;
      sizer.margin = 6;
      sizer.spacing = 4;
      sizer.add( this.sourceHaImage_Sizer);
      sizer.add( this.sourceHaControl );
      sizer.add( this.sourceHaMultiplicationControl );
      sizer.add( this.sourceRGBImage_Sizer);
      sizer.add( this.sourceRGBControl );
      sizer.add( this.Rescale_CheckBox );

  }


   this.CCBgImage_Label = new Label( this );
   this.CCBgImage_Label.minWidth = labelWidth1; // align with labels inside group boxes below
   this.CCBgImage_Label.text = "Background Preview:";
   this.CCBgImage_Label.textAlignment = TextAlign_Right|TextAlign_VertCenter;

   this.CCBgImage_ViewList = new ViewList( this );
   this.CCBgImage_ViewList.minWidth = 200;
   this.CCBgImage_ViewList.getAll(); // include main views as well as previews
   this.CCBgImage_ViewList.toolTip = "Select the preview for background neutralization.";
   this.CCBgImage_ViewList.onViewSelected = function( view )
   {
      data.Changed=true;
      data.CCBgView = view;
   };

   this.CCBgImage_Sizer = new HorizontalSizer;
   this.CCBgImage_Sizer.spacing = 4;
   this.CCBgImage_Sizer.add( this.CCBgImage_Label );
   this.CCBgImage_Sizer.add( this.CCBgImage_ViewList, 100 );



   this.CCWhiteImage_Label = new Label( this );
   this.CCWhiteImage_Label.minWidth = labelWidth1; // align with labels inside group boxes below
   this.CCWhiteImage_Label.text = "White Balance Preview:";
   this.CCWhiteImage_Label.textAlignment = TextAlign_Right|TextAlign_VertCenter;

   this.CCWhiteImage_ViewList = new ViewList( this );
   this.CCWhiteImage_ViewList.minWidth = 200;
   this.CCWhiteImage_ViewList.getAll(); // include main views as well as previews
   this.CCWhiteImage_ViewList.toolTip = "Select the preview for white balance.";
   this.CCWhiteImage_ViewList.onViewSelected = function( view )
   {
      data.Changed=true;
      data.CCWhiteView = view;
   };

   this.CCWhiteImage_Sizer = new HorizontalSizer;
   this.CCWhiteImage_Sizer.spacing = 4;
   this.CCWhiteImage_Sizer.add( this.CCWhiteImage_Label );
   this.CCWhiteImage_Sizer.add( this.CCWhiteImage_ViewList, 100 );


   this.StructureDetection_CheckBox = new CheckBox( this );
   this.StructureDetection_CheckBox.text = "Structure Detection";
   this.StructureDetection_CheckBox.checked = data.StructureDetection;
   this.StructureDetection_CheckBox.toolTip =
      "<p>If this option is selected, the script will calibrate final image "
      + "with the small-scales layers selected.</p>";
   this.StructureDetection_CheckBox.onCheck = function( checked )
   {
      data.Changed=true;
      data.StructureDetection = checked;
   };

   this.numberOfLayers_Label = new Label( this );
   this.numberOfLayers_Label.minWidth = 100;
   this.numberOfLayers_Label.text = "Structure Layers :";
   this.numberOfLayers_Label.textAlignment = TextAlign_Right|TextAlign_VertCenter;

   this.numberOfLayers_SpinBox = new SpinBox( this );
   this.numberOfLayers_SpinBox.minValue = 1;
   this.numberOfLayers_SpinBox.maxValue = 8;
   this.numberOfLayers_SpinBox.value = data.numberOfLayers;
   this.numberOfLayers_SpinBox.toolTip = this.numberOfLayers_Label.toolTip =
      "<b>Number of wavelet layers that will be used to calibrate final image.</b>";
   this.numberOfLayers_SpinBox.onValueUpdated = function( value )
   {
      data.Changed=true;
      data.numberOfLayers = value;
   };


   this.SCNR_CheckBox = new CheckBox( this );
   this.SCNR_CheckBox.text = "SCNR to Green";
   this.SCNR_CheckBox.checked = data.TSCNR;
   this.SCNR_CheckBox.toolTip =
      "<p>If this option is selected, the script will perfom an SCNR "
      + "correction to final image.</p>";
   this.SCNR_CheckBox.onCheck = function( checked )
   {
      data.Changed=true;
      data.TSCNR = checked;
   };

   this.CC_CheckBox = new CheckBox( this );
   this.CC_CheckBox.text = "Apply Color Calibration";
   this.CC_CheckBox.checked = data.CC;
   this.CC_CheckBox.toolTip =
      "<p>If this option is selected, the script will perfom a  Color Calibration "
      + "to final image.</p>";
   this.CC_CheckBox.onCheck = function( checked )
   {
      data.Changed=true;
      data.CC = checked;
   };



   this.numberOfLayers_Sizer = new HorizontalSizer;
   this.numberOfLayers_Sizer.setAlignment = Align_Center;
   this.numberOfLayers_Sizer.spacing = 4;
   this.numberOfLayers_Sizer.add( this.StructureDetection_CheckBox );
   this.numberOfLayers_Sizer.add( this.numberOfLayers_Label );
   this.numberOfLayers_Sizer.add( this.numberOfLayers_SpinBox );
   this.numberOfLayers_Sizer.add( this.SCNR_CheckBox );
   this.numberOfLayers_Sizer.add( this.CC_CheckBox );
   this.numberOfLayers_Sizer.addStretch();


this.CCGroupBox = new GroupBox( this );
   with ( this.CCGroupBox )
   {

      title = "Color Calibration Control";
      sizer = new VerticalSizer;
      sizer.margin = 6;
      sizer.spacing = 4;
      sizer.add( this.CCBgImage_Sizer);
      sizer.add( this.CCWhiteImage_Sizer);
      sizer.add( this.numberOfLayers_Sizer);

  }



   this.STF_CheckBox = new CheckBox( this );
   this.STF_CheckBox.text = "AutoSTF";
   this.STF_CheckBox.checked = data.CSTF;
   this.STF_CheckBox.toolTip =
      "<p>If this option is selected, the script will perfom an AutoSTF "
      + "to final image.</p>";
   this.STF_CheckBox.onCheck = function( checked )
   {
      //data.Changed=true;
      data.CSTF = checked;
   };



   this.STFGroupBox = new GroupBox( this );
   this.STFGroupBox.title = "Screen Transfer Function";
   this.STFGroupBox.sizer = new VerticalSizer;
      with ( this.STFGroupBox.sizer )
   {
      margin = 6;
      spacing = 4;
      addStretch();
      add( this.STF_CheckBox);
     ;
   }

   this.showpreview_Button = new PushButton( this );
   with ( this.showpreview_Button )
   {
      text = "Show HaRGB";
      onClick = function()
      {
         parent.parent.showHaRGB();
      }
   }

   this.showoriginal_Button = new PushButton( this );
   with ( this.showoriginal_Button )
   {
      text = "Show RGB";
      onClick = function()
      {
         parent.parent.showOriginal();
      }
   }


   this.PreviewGroupBox = new GroupBox( this );
   this.PreviewGroupBox.title = "Preview buttons";
   this.PreviewGroupBox.sizer = new HorizontalSizer;
      with ( this.PreviewGroupBox.sizer )
   {
      margin = 6;
      spacing = 4;
      addStretch();
      add( this.showoriginal_Button);
      add( this.showpreview_Button);
   }

   this.buttons = new HorizontalSizer;
   this.buttons.addStretch();
   this.buttons.add( this.cancelButton );
   this.buttons.add( this.okButton );

   this.tabsbuttons = new VerticalSizer;
   this.tabsbuttons.setAlignment = Align_Top;
   this.tabsbuttons.margin = 6;
   this.tabsbuttons.spacing = 6;

   this.tabsbuttons.addStretch();
   this.tabsbuttons.add( this.ChannelsGroupBox);
   this.tabsbuttons.add( this.CCGroupBox);
   this.tabsbuttons.add( this.STFGroupBox);
   this.tabsbuttons.add( this.PreviewGroupBox);
   this.tabsbuttons.add( this.buttons);


   this.sizer = new VerticalSizer;
   this.sizer.margin = 6;
   this.sizer.spacing = 6;
   this.sizer.add( this.PWGroupBox );
   this.sizer.add( this.tabsbuttons );

   this.windowTitle = "HaRGB Combination Script";
   this.adjustToContents();
}

MyDialog.prototype = new Dialog;



var window = ImageWindow.activeWindow;
var data = new HaRGBCombinationData;


var dlg = new MyDialog;
console.hide();
dlg.execute();
